<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns:xd="http://www.pnp-software.com/XSLTdoc"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:s="http://www.ascc.net/xml/schematron"
    xmlns:exsl="http://exslt.org/common"
    extension-element-prefixes="exsl"
    xmlns:xsp="http://apache.org/xsp/core/v1"
    xmlns:xi="http://www.w3.org/2001/XInclude"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" 
    xmlns:a="http://relaxng.org/ns/compatibility/annotations/1.0" 
    xmlns:f="http://axkit.org/NS/xsp/perform/v1" 
    xmlns:tei="http://www.tei-c.org/ns/1.0" 
    exclude-result-prefixes="exsl rng a f tei s xs xd" 
    xmlns:rng="http://relaxng.org/ns/structure/1.0" 
    version="1.0">

<xd:doc type="stylesheet">
    <xd:short>
      TEI stylesheet for simplifying Relax NG markup
      </xd:short>
    <xd:detail>
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      </xd:detail>
    <xd:author>Sebastian Rahtz sebastian.rahtz@oucs.ox.ac.uk</xd:author>
    <xd:cvsId>$Id: tei ,v 5.0.0 2005/04/12 10:13:06 rahtz Exp $</xd:cvsId>
    <xd:copyright>2005, TEI Consortium</xd:copyright>
  </xd:doc>

<xsl:output indent="yes" encoding="utf-8"/>

<xsl:key  name="DEFINES" match="rng:define" use="@name"/>


<xsl:template match="*|@*|text()|comment()">
  <xsl:copy>
    <xsl:apply-templates select="*|@*|text()|comment()"/>
  </xsl:copy>
</xsl:template>



<xsl:template
    match="rng:define">
  <xsl:choose>
    <xsl:when test="count(rng:*)=1 and rng:choice/rng:notAllowed">
    </xsl:when>
    <xsl:otherwise>
      <define xmlns="http://relaxng.org/ns/structure/1.0">
	<xsl:copy-of select="@*"/>
	<xsl:apply-templates/>
      </define>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template
    match="rng:choice|rng:list|rng:group|rng:oneOrMore|rng:optional|rng:zeroOrMore">
  <xsl:call-template name="simplifyRelax">
    <xsl:with-param name="element">
      <xsl:value-of select="local-name(.)"/>
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>

<xsl:template name="simplifyRelax">
<!-- 
for each Relax NG content model,
remove reference to any elements which have been
deleted, or to classes which are empty.
This may make the container empty,
so that is only put back in if there is some content
-->
  <xsl:param name="element"/>
  <xsl:variable name="contents">
    <WHAT>
      <xsl:for-each select="rng:*">
	<xsl:choose>
	  <xsl:when test="local-name(.)='attribute'">
	      <xsl:copy-of select="."/>
	  </xsl:when>
	  <xsl:when test="local-name(.)='data'">
	    <data xmlns="http://relaxng.org/ns/structure/1.0">
	      <xsl:copy-of select="@*"/>
	    </data>
	  </xsl:when>
	  <xsl:when test="local-name(.)='text'">
	    <text xmlns="http://relaxng.org/ns/structure/1.0"/>
	  </xsl:when>
	  <xsl:when test="local-name(.)='value'">
	    <value xmlns="http://relaxng.org/ns/structure/1.0">
	      <xsl:apply-templates/>
	    </value>
	  </xsl:when>
	  <xsl:when test="local-name(.)='ref'">
	    <xsl:choose>
		<xsl:when test="key('DEFINES',@name)">
	          <xsl:for-each select="key('DEFINES',@name)">
		    <xsl:choose>
		      <xsl:when test="count(rng:*)=1 and rng:choice/rng:notAllowed">
		      </xsl:when>
		      <xsl:otherwise>
			<ref name="{@name}" xmlns="http://relaxng.org/ns/structure/1.0"/>
		      </xsl:otherwise>
		    </xsl:choose>
		  </xsl:for-each>
		</xsl:when>
		<xsl:otherwise>
		  <xsl:message>Cant find <xsl:value-of select="@name"/></xsl:message>
		</xsl:otherwise>
	    </xsl:choose>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:call-template name="simplifyRelax">
	      <xsl:with-param name="element"><xsl:value-of select="local-name(.)"/></xsl:with-param>
	    </xsl:call-template>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:for-each>
    </WHAT>
  </xsl:variable>
  <xsl:variable name="entCount">
    <xsl:for-each select="exsl:node-set($contents)/WHAT">
      <xsl:value-of select="count(*)"/>
    </xsl:for-each>
  </xsl:variable>
  <xsl:if test="$entCount&gt;0">
    <xsl:element name="{$element}" xmlns="http://relaxng.org/ns/structure/1.0">
      <xsl:copy-of select="exsl:node-set($contents)/WHAT/*"/>
    </xsl:element>
  </xsl:if>
</xsl:template>

</xsl:stylesheet>

